/*####################################################################
 FILE: lettura_dati_ina219.ino
 VERSION: 1.0
 Descrizione: Programma di minitester con utilizzo dei seguenti componenti
 Adafruit https://www.adafruit.com/
 
 INA219  High Side DC Current Sensor Breakout - 26V ±3.2A Max [ID:904]
 Tutorials: [[http://www.adafruit.com/products/904#tutorials]]
 
 Monochrome 128x32 I2C OLED graphic display [ID:931]
 Tutorials: http://www.adafruit.com/products/931#tutorials
  
 Ultima modifica il 22/5/2014
 Applicazione realizzata da Adriano Gandolfo
 Sito http://www.adrirobot.it 
 Blog http://it.emcelettronica.com/blog/26920
 
 This example code is in the public domain.
 #################################################################### */

//Caricamento delle librerie di comunicazione
#include <SPI.h>
#include <Wire.h>
//Caricamento delle librerie gestione display
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define OLED_RESET 4 //Definizione pin reset display
Adafruit_SSD1306 display(OLED_RESET);
//Caricamento delle librerie gestione sensore 
#include <Adafruit_INA219.h>
//Costruisce un'istanza della Adafruit_INA219
//Se non viene specificato alcun indirizzo, viene utilizzato
//quello di default (0x40). Se si collegano più modulo INA219
//occorre indicare il suo indirizzo per esempio
// Adafruit_INA219 ina219_B(0x41)
Adafruit_INA219 ina219;

void setup()   {                
  //Impostazione modulo INA219
  uint32_t currentFrequency;
  //Inizializza la comunicazione I2C con il dispositivo Adafruit_INA219
  // utilizzando i valori di configurazione predefiniti.
  ina219.begin();
  // Impostazione modulo display con indirizzo I2C addr 0x3C
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  // Cancellazione schermo e buffer
  display.clearDisplay();
  // Messaggio iniziale
  display.setTextSize(1); //Definizione della grandezza del carattere
  display.setTextColor(WHITE);  //Impostazione del colore del testo
  display.setCursor(0,0); //Posizionamento del cursore
  display.println("www.adrirobot.it");
  display.setTextColor(BLACK, WHITE); // Inverte il colore del testo
  display.println("Voltmetro digitale");
  display.display(); // Stampa sul display
  delay(2000); //Pausa per lettura dei valori
}

void loop()
{
  //Definizione variabili
  float shuntvoltage = 0;
  float busvoltage = 0;
  float current_mA = 0;
  float loadvoltage = 0;
  //Acquisizione dei valori dal sensore
  shuntvoltage = ina219.getShuntVoltage_mV();
  //Legge la tensione tra V e V +. Questa è la caduta di tensione
  //misurata ai capi del resistore shunt. Il valore è espresso in mV
  busvoltage = ina219.getBusVoltage_V();
  //Legge la tensione tra GND e V. Questa è la tensione totale vista
  //dal circuito in esame. (Tensione di alimentazione - tensione shunt). 
  //Il valore è in Volt.
  current_mA = ina219.getCurrent_mA();
  //Legge la corrente, ricavata tramite la legge di Ohm
  //dalla tensione shunt misurata. Il valore è in mA.
  loadvoltage = busvoltage + (shuntvoltage / 1000);
  //Calcola la tensione sul carico
  //Stampa dei valori misurati sul display
  display.clearDisplay(); // cancellazione dello schermo
  display.setTextSize(2); //Definizione della grandezza del testo
  display.setTextColor(WHITE); //Impostazione del colore del testo
  display.setCursor(0,0); //Posizionamento del cursore
  // Definizione dei valori con relative unità di misura
  display.print("V:"); 
  display.print(loadvoltage, 2); 
  display.println(" V");
  display.print("A:"); 
  display.print(current_mA,1); 
  display.println(" mA");
  display.display(); // Stampa dei valori sul display
  delay(2000); //Pausa per lettura dei valori
  display.clearDisplay(); // cancellazione dello schermo
  display.setCursor(0,0);  //Posizionamento del cursore
  // Mostra sul display il valore della potenza assorbita
  // ottenuta moltiplicando la corrente per la tensione sul carico
  display.println("Potenza");
  display.print((current_mA*loadvoltage)/1000);  
  display.print(" W");
  display.display(); // Stampa dei valori sul display
  delay(2000); //Pausa per lettura dei valori
}



